from src.models.abstract_reference import abstract_reference
from src.models.measurement_unit_model import measurement_unit_model
from src.models.nomenclature_group_model import nomenclature_group_model
from src.validation.validator import validator

class nomenclature_model(abstract_reference):
    # Полное наименование
    __full_name = ""
    # Единица измерения
    __measurement_unit = None
    # Группа номенклатуры
    __nomenclature_group = None

    # Валидатор
    __vtor = validator()


    def __init__(self, name, full_name, measurement_unit, nomenclature_group):
        """
        Args:
            full_name (str): Полное наименование (до 256 символов)
            measurement_unit (measurement_unit_model): Единица измерения
            nomenclature_group (nomenclature_group_model): Группа номенклатуры
        """

        self.__vtor.check_type(full_name, str)
        self.__vtor.check_length_less(full_name, 256)
        self.__vtor.check_type(measurement_unit, measurement_unit_model)
        self.__vtor.check_type(nomenclature_group, nomenclature_group_model)

        self.__full_name = full_name
        self.__measurement_unit = measurement_unit
        self.__nomenclature_group = nomenclature_group

        super().__init__(name)

    @property
    def full_name(self):
        """Полное наименование (до 256 символов)"""
        
        return self.__full_name
    
    @full_name.setter
    def full_name(self, value: str):
        """
        Полное наименование (до 256 символов)
        Args:
            value (str): Полное наименование
        Raises:
            argument_exception: Несоответствие типа аргумента
            argument_exception: Превышена длина аргумента
        """

        self.__vtor.check_type(value, str)
        self.__vtor.check_length_less(256)

        self.__full_name = value
    
    @property
    def measurement_unit(self):
        """Единица измерения"""

        return self.__measurement_unit
    
    @measurement_unit.setter
    def measurement_unit(self, value: measurement_unit_model):
        """
        Единица измерения
        Args:
            value (measurement_unit_model): Единица измерения
        Raises:
            argument_exception: Несоответствие типа аргумента
        """

        self.__vtor.check_type(value, measurement_unit_model)

        self.__measurement_unit = value
    
    @property
    def nomenclature_group(self):
        """Группа номенклатуры"""

        return self.__nomenclature_group
    
    @nomenclature_group.setter
    def nomenclature_group(self, value: nomenclature_group_model):
        """
        Группа номенклатуры
        Args:
            value (nomenclature_group_model): Группа номенклатуры
        Raises:
            argument_exception: Несоответствие типа аргумента
        """

        self.__vtor.check_type(value, nomenclature_group_model)

        self.__nomenclature_group = value